@@ -12,17 +12,18 @@ contains
     subroutine sub_1(type_1, i)
         class(t_2), intent(inout) :: type_1
         integer, intent(in) :: i
-        allocate(type_1%type_2(0))
-        associate(target=>type_1%type_2(0)%i)
+        allocate(type_1%type_2(1))
+        associate(target=>type_1%type_2(1)%i)
             target = i
         end associate
     end subroutine sub_1
 
     subroutine sub_2(progress)
         class(t_2), intent(inout), allocatable :: progress(:)
-        allocate(progress(0)%type_2(0))
-        associate(target => progress(0))
-            target%type_2(0)%i = 345
+        allocate(progress(1))
+        allocate(progress(1)%type_2(1))
+        associate(target => progress(1))
+            target%type_2(1)%i = 345
         end associate
     end subroutine sub_2
 end module
@@ -31,6 +32,12 @@ program associate2
     use associate_08_module_1
     implicit none
     type(t_2) :: type_1
+    class(t_2), allocatable :: t_2_array(:)
     call sub_1(type_1, 123)
-    print *, type_1%type_2(0)%i
+    if (type_1%type_2(1)%i /= 123) error stop
+    print *, type_1%type_2(1)%i
+
+    ! TODO: Fix when implementing class arrays
+    ! call sub_2(t_2_array)
+    ! print *, t_2_array(1)%type_2(1)%i
 end program associate2
