@@ -3,6 +3,7 @@ module class_procedure_args_01
    type,public :: clock
    contains
       procedure,public :: toc => clock_end
+      procedure :: increment
    end type clock
 
    type(clock),public :: clk
@@ -22,6 +23,12 @@ contains
       if (case_str /= "6") error stop
    end subroutine sub
 
+   function increment(me, x) result(res)
+      class(clock),intent(inout) :: me
+      integer, intent(in) :: x
+      integer :: res
+      res = x + 1
+   end function increment
 end module class_procedure_args_01
 
 
@@ -34,6 +41,6 @@ program main
 
    toc => sub
    call toc("6")
-
+   if (clk%increment(x=5) /= 6) error stop
 end program main
 
