@@ -34,7 +34,17 @@ program intrinsics_179
     real :: exp_res_aaaa_4(4, 5, 6)
 
     aa = 1.0
-    mask = .true.
+    ! TODO: Uncomment after https://github.com/lfortran/lfortran/issues/7412 is fixed.
+    ! mask = .true.
+    do i = lbound(mask, 1), ubound(mask, 1)
+    do j = lbound(mask, 2), ubound(mask, 2)
+    do k = lbound(mask, 3), ubound(mask, 3)
+    do l = lbound(mask, 4), ubound(mask, 4)
+    mask(i, j, k, l) = .true.
+    end do
+    end do
+    end do
+    end do
 
     do i = 1, 4
         do j = 1, 5
@@ -294,8 +304,18 @@ program intrinsics_179
     end do
 
     print *, sum(sum(aaaa, dim, mask))
-    if (abs(sum(sum(aaaa, dim)) - 2.27976032e-02) > 1e-8) error stop
-    mask = .false.
+    if (abs(sum(sum(aaaa, dim, mask)) - 2.27976032e-02) > 1e-8) error stop
+    ! TODO: Uncomment after https://github.com/lfortran/lfortran/issues/7412 is fixed.
+    ! mask = .false.
+    do i = lbound(mask, 1), ubound(mask, 1)
+    do j = lbound(mask, 2), ubound(mask, 2)
+    do k = lbound(mask, 3), ubound(mask, 3)
+    do l = lbound(mask, 4), ubound(mask, 4)
+    mask(i, j, k, l) = .false.
+    end do
+    end do
+    end do
+    end do
     print *, sum(sum(aaaa, dim, mask))
     if (abs(sum(sum(aaaa, dim, mask)) - 0.0) > 1e-8) error stop
     mask(1, 1, :, :) = .true.
