@@ -3,7 +3,9 @@ module nested_vars3
 
     contains
 
-    subroutine h()
+    subroutine h(a, filename)
+        integer, intent(in) :: a
+        character(len=*), intent(in) :: filename
         integer, parameter :: x = 10
         call g()
 
@@ -12,6 +14,9 @@ module nested_vars3
         subroutine g()
             print *, x
             if (x /= 10) error stop
+            print *, a
+            if (a /= 5) error stop
+            if (filename /= "xyz") error stop
         end subroutine
 
     end subroutine
@@ -22,5 +27,5 @@ end module
 program nested_vars3_main
     use nested_vars3, only: h
     implicit none
-    call h()
+    call h(5, "xyz")
 end program
