@@ -13,4 +13,13 @@ program shifta_02
     result = shifta(n, x)
     print *, "shifta(", n, ", ", x, ") = ", result
     if (result /= 0) error stop
+
+block
+        integer(8) :: x = 32
+        integer :: y = 2
+        result = shifta(x, y)
+        print *, "shifta(", x, ", ", y, ") = ", result
+        if (result /= 8) error stop "Test case failed: shifta type mismatch"
+end block
+
 end program
