@@ -5,14 +5,19 @@ real(4), allocatable :: x0(:)
 real(4), allocatable :: xl(:)
 real(4), allocatable :: xu(:)
 real(4) :: rhobeg
+integer :: i
 
-rhobeg = 1.0
+rhobeg = 0.5
 
 allocate(x0(5), xl(5), xu(5))
 
-xl = 2.0
+xl = 0.5
 xu = 3.0
 
+do i = lbound(x0, 1), ubound(x0, 1)
+    x0 = i
+end do
+
 where (x0 <= xl + 0.5 * rhobeg)
     x0 = xl
 elsewhere(x0 < xl + rhobeg)
@@ -26,6 +31,6 @@ elsewhere(x0 > xu - rhobeg)
 end where
 
 print *, x0
-if( any(x0 /= [2.0, 2.0, 2.0, 2.0, 2.0]) ) error stop
+if( any(x0 /= [3.0, 3.0, 3.0, 3.0, 3.0]) ) error stop
 
 end program
