@@ -1,4 +1,6 @@
+#include <exception>
 #include <filesystem>
+#include <random>
 #include <regex>
 #include <stdexcept>
 #include <thread>
@@ -15,8 +17,9 @@
 #endif // CLI11_HAS_FILESYSTEM
 #include <bin/CLI11.hpp>
 
+#include <bin/concurrent_lfortran_lsp_language_server.h>
 #include <bin/language_server_interface.h>
-#include <bin/lfortran_lsp_language_server.h>
+#include <bin/parallel_lfortran_lsp_language_server.h>
 
 namespace LCompilers::LLanguageServer::Interface {
     namespace fs = std::filesystem;
@@ -26,20 +29,16 @@ namespace LCompilers::LLanguageServer::Interface {
 
     auto validateConfigSection(const std::string &configSection) -> std::string {
         if (!std::regex_match(configSection, RE_CONFIG_SECTION)) {
-            throw lc::LCompilersException(
-                ("Configuration section is not a valid sequence of dot-separated, "
-                 "ECMAScript Ids: " + configSection)
-            );
+            return ("Configuration section is not a valid sequence of dot-separated, "
+                    "ECMAScript Ids: " + configSection) ;
         }
 
         if (std::regex_match(configSection, RE_RESERVED_CONFIG_IDS)) {
-            throw lc::LCompilersException(
-                ("Configuration section contains a reserved Id for ECMAScript: " +
-                 configSection)
-            );
+            return ("Configuration section contains a reserved Id for ECMAScript: " +
+                    configSection);
         }
 
-        return configSection;
+        return "";
     }
 
     auto existsAndIsWritable(const std::string &pathString) -> std::string {
@@ -175,6 +174,27 @@ namespace LCompilers::LLanguageServer::Interface {
         );
     }
 
+    const std::map<ExecutionStrategy, std::string> ExecutionStrategyNames = {
+        {ExecutionStrategy::PARALLEL, "PARALLEL"},
+        {ExecutionStrategy::CONCURRENT, "CONCURRENT"},
+    };
+
+    const std::map<ExecutionStrategy, std::string> ExecutionStrategyValues {
+        {ExecutionStrategy::PARALLEL, "parallel"},
+        {ExecutionStrategy::CONCURRENT, "concurrent"},
+    };
+
+    auto executionStrategyByValue(const std::string &value) -> ExecutionStrategy {
+        for (const auto &[strategy_key, strategy_value] : ExecutionStrategyValues) {
+            if (strategy_value == value) {
+                return strategy_key;
+            }
+        }
+        throw std::invalid_argument(
+            "Invalid ExecutionStrategy value: \"" + value + "\""
+        );
+    }
+
     LanguageServerInterface::LanguageServerInterface()
         : workspaceConfig(std::make_shared<lsc::LFortranLspConfig>())
     {
@@ -231,6 +251,17 @@ namespace LCompilers::LLanguageServer::Interface {
             )
         );
 
+        opts.executionStrategy = ExecutionStrategy::CONCURRENT;
+        server->add_option(
+            "--execution-strategy", opts.executionStrategy,
+            "Specifies the execution strategy for handling messages. The `parallel` strategy implies multiple messages may be processed alongside each other, while the `concurrent` strategy implies multiple messages may be processed but only one processor will be active at a time (they will yield control to each other)."
+        )->capture_default_str()->transform(
+            CLI::CheckedTransformer(
+                transpose(ExecutionStrategyValues),
+                CLI::ignore_case
+            )
+        );
+
         opts.numRequestThreads = 5;
         server->add_option(
             "--num-request-threads", opts.numRequestThreads,
@@ -243,6 +274,12 @@ namespace LCompilers::LLanguageServer::Interface {
             "Number of threads that will handle sub-tasks of requests."
         )->capture_default_str();
 
+        opts.enableLogging = false;
+        server->add_flag(
+            "--enable-logging", opts.enableLogging,
+            "Enable logging to a file."
+        )->capture_default_str();
+
         opts.configSection = "LFortran";
         server->add_option(
             "--config-section", opts.configSection,
@@ -285,7 +322,11 @@ namespace LCompilers::LLanguageServer::Interface {
             "Additional flags to pass to the LFortran compiler."
         );
 
-        workspaceConfig->log.path = existsAndIsWritable("lfortran-language-server.log");
+        if (opts.enableLogging) {
+            workspaceConfig->log.path = existsAndIsWritable("lfortran-language-server.log");
+        } else {
+            workspaceConfig->log.path = "";
+        }
         server->add_option(
             "--log-path", workspaceConfig->log.path,
             "Path to where logs should be written."
@@ -314,12 +355,54 @@ namespace LCompilers::LLanguageServer::Interface {
             "Number of spaces per level of indentation."
         )->capture_default_str();
 
+        workspaceConfig->timeoutMs = 100;
+        server->add_option(
+            "--timeout-ms", workspaceConfig->timeoutMs,
+            "Number of milliseconds to await requests from server-to-client."
+        )->capture_default_str();
+
+        workspaceConfig->retry.maxAttempts = 5;
+        server->add_option(
+            "--max-retry-attempts", workspaceConfig->retry.maxAttempts,
+            "Maximum number of times to attempt a request before giving up."
+        )->capture_default_str();
+
+        workspaceConfig->retry.minSleepTimeMs = 10;
+        server->add_option(
+            "--min-retry-sleep-time-ms", workspaceConfig->retry.minSleepTimeMs,
+            "Minimum number of milliseconds to wait between request attempts."
+        )->capture_default_str();
+
+        workspaceConfig->retry.maxSleepTimeMs = 10;
+        server->add_option(
+            "--max-retry-sleep-time-ms", workspaceConfig->retry.maxSleepTimeMs,
+            "Maximum number of milliseconds to wait between request attempts."
+        )->capture_default_str();
+
+        workspaceConfig->telemetry.enabled = false;
+        server->add_option(
+            "--telemetry-enabled", workspaceConfig->telemetry.enabled,
+            "Whether to enable telemetry events (may require a restart)."
+        )->capture_default_str();
+
+        workspaceConfig->telemetry.frequencyMs = 1000;
+        server->add_option(
+            "--telemetry-frequency-ms", workspaceConfig->telemetry.frequencyMs,
+            "Number of milliseconds to wait between telemetry events."
+        )->capture_default_str();
+
         opts.extensionId = "lcompilers.lfortran-lsp";
         server->add_option(
             "--extension-id", opts.extensionId,
             "Identifies the language client extension that interacts with this server."
         )->capture_default_str();
 
+        opts.parentProcessId = -1;
+        server->add_option(
+            "--parent-process-id", opts.parentProcessId,
+            "Process ID (PID) of the process that has spawned this language server."
+        )->capture_default_str();
+
         return server;
     }
 
@@ -342,38 +425,73 @@ namespace LCompilers::LLanguageServer::Interface {
     auto LanguageServerInterface::buildLanguageServer(
         ls::MessageQueue &incomingMessages,
         ls::MessageQueue &outgoingMessages,
-        lsl::Logger &logger
+        lsl::Logger &logger,
+        std::atomic_bool &start,
+        std::condition_variable &startChanged,
+        std::mutex &startMutex
     ) -> std::unique_ptr<ls::LanguageServer> {
         if (opts.language == Language::FORTRAN) {
             if (opts.dataFormat == DataFormat::JSON_RPC) {
                 if (opts.serverProtocol == ServerProtocol::LSP) {
-                    return std::make_unique<lsp::LFortranLspLanguageServer>(
-                        incomingMessages,
-                        outgoingMessages,
-                        opts.numRequestThreads,
-                        opts.numWorkerThreads,
-                        logger,
-                        opts.configSection,
-                        opts.extensionId,
-                        LFORTRAN_VERSION,
-                        workspaceConfig
-                    );
+                    switch (opts.executionStrategy) {
+                    case ExecutionStrategy::PARALLEL: {
+                        std::random_device randomSeed;
+                        return std::make_unique<lsp::ParallelLFortranLspLanguageServer>(
+                            incomingMessages,
+                            outgoingMessages,
+                            opts.numRequestThreads,
+                            opts.numWorkerThreads,
+                            logger,
+                            opts.configSection,
+                            opts.extensionId,
+                            LFORTRAN_VERSION,
+                            opts.parentProcessId,
+                            randomSeed(),
+                            workspaceConfig,
+                            start,
+                            startChanged,
+                            startMutex
+                        );
+                        break;
+                    }
+                    case ExecutionStrategy::CONCURRENT: {
+                        return std::make_unique<lsp::ConcurrentLFortranLspLanguageServer>(
+                            incomingMessages,
+                            outgoingMessages,
+                            logger,
+                            opts.configSection,
+                            opts.extensionId,
+                            LFORTRAN_VERSION,
+                            opts.parentProcessId,
+                            workspaceConfig,
+                            start,
+                            startChanged,
+                            startMutex
+                        );
+                        break;
+                    }
+                    default: {
+                        throw lc::LCompilersException(
+                            ("Unsupported execution strategy: " +
+                             ExecutionStrategyValues.at(opts.executionStrategy))
+                        );
+                    }
+                    }
                 } else {
                     throw lc::LCompilersException(
                         ("Unsupported server protocol for fortran: " +
-                         std::to_string(static_cast<int>(opts.serverProtocol)))
+                         ServerProtocolValues.at(opts.serverProtocol))
                     );
                 }
             } else {
                 throw lc::LCompilersException(
                     ("Unsupported data format for fortran: " +
-                     std::to_string(static_cast<int>(opts.dataFormat)))
+                     DataFormatValues.at(opts.dataFormat))
                 );
             }
         } else {
             throw lc::LCompilersException(
-                ("Unsupported language: " +
-                 std::to_string(static_cast<int>(opts.language)))
+                ("Unsupported language: " + LanguageValues.at(opts.language))
             );
         }
     }
@@ -383,7 +501,10 @@ namespace LCompilers::LLanguageServer::Interface {
         ls::MessageStream &messageStream,
         ls::MessageQueue &incomingMessages,
         ls::MessageQueue &outgoingMessages,
-        lsl::Logger &logger
+        lsl::Logger &logger,
+        std::atomic_bool &start,
+        std::condition_variable &startChanged,
+        std::mutex &startMutex
     ) -> std::unique_ptr<ls::CommunicationProtocol> {
         switch (opts.communicationProtocol) {
         case CommunicationProtocol::STDIO: {
@@ -392,7 +513,10 @@ namespace LCompilers::LLanguageServer::Interface {
                 messageStream,
                 incomingMessages,
                 outgoingMessages,
-                logger
+                logger,
+                start,
+                startChanged,
+                startMutex
             );
         }
         default: {
@@ -404,47 +528,63 @@ namespace LCompilers::LLanguageServer::Interface {
         }
     }
 
-    auto LanguageServerInterface::buildMessageQueue(
-        lsl::Logger &logger
-    ) -> std::unique_ptr<ls::MessageQueue> {
-        return std::make_unique<ls::MessageQueue>(logger);
-    }
-
     auto LanguageServerInterface::serve() -> void {
         try {
-            lsl::Logger logger(workspaceConfig->log.path);
-            try {
-                std::unique_ptr<ls::MessageStream> messageStream =
-                    buildMessageStream(logger);
-                ls::MessageQueue communicatorToServer(logger);
-                ls::MessageQueue serverToCommunicator(logger);
-                std::unique_ptr<ls::LanguageServer> languageServer =
-                    buildLanguageServer(
-                        communicatorToServer,
-                        serverToCommunicator,
-                        logger);
-                std::unique_ptr<ls::CommunicationProtocol> communicationProtocol =
-                    buildCommunicationProtocol(
-                        *languageServer,
-                        *messageStream,
-                        serverToCommunicator,
-                        communicatorToServer,
-                        logger
-                    );
-                communicationProtocol->serve();
-                logger.debug()
-                    << "[LanguageServerInterface] Communication protocol terminated."
-                    << std::endl;
-                logger.info() << "Language server terminated cleanly.";
-            } catch (const std::exception &e) {
-                std::string buffer = "Language server terminated erroneously: ";
-                buffer.append(e.what());
-                throw lc::LCompilersException(buffer);
+            lsl::Logger logger(opts.enableLogging ? workspaceConfig->log.path : "", "LanguageServerInterface");
+            logger.setLevel(workspaceConfig->log.level);
+            logger.threadName("main");
+
+            std::atomic_bool start{false};
+            std::condition_variable startChanged;
+            std::mutex startMutex;
+
+            std::unique_ptr<ls::MessageStream> messageStream =
+                buildMessageStream(logger);
+            ls::MessageQueue communicatorToServer(logger, "communicator-to-server");
+            ls::MessageQueue serverToCommunicator(logger, "server-to-communicator");
+            std::unique_ptr<ls::LanguageServer> languageServer =
+                buildLanguageServer(
+                    communicatorToServer,
+                    serverToCommunicator,
+                    logger,
+                    start,
+                    startChanged,
+                    startMutex
+                );
+            std::unique_ptr<ls::CommunicationProtocol> communicationProtocol =
+                buildCommunicationProtocol(
+                    *languageServer,
+                    *messageStream,
+                    serverToCommunicator,
+                    communicatorToServer,
+                    logger,
+                    start,
+                    startChanged,
+                    startMutex
+                );
+            {
+                std::unique_lock<std::mutex> startLock(startMutex);
+                start = true;
             }
+            startChanged.notify_all();
+            communicationProtocol->serve();
+            logger.info()
+                << "Language server terminated cleanly."
+                << std::endl;
+        } catch(const LCompilers::LCompilersException &e) {
+            std::cerr << "Caught unhandled exception" << std::endl;
+            std::vector<LCompilers::StacktraceItem> d = e.stacktrace_addresses();
+            get_local_addresses(d);
+            get_local_info(d);
+            std::cerr << stacktrace2str(d, LCompilers::stacktrace_depth, false);
+            std::cerr << e.name() + ": " << e.msg() << std::endl;
+            throw e;
         } catch (const std::exception &e) {
             std::string buffer = "Caught unhandled exception: ";
             buffer.append(e.what());
             throw lc::LCompilersException(buffer);
+        } catch (...) {
+            std::cerr << "Unknown Exception" << std::endl;
         }
     }
 
