@@ -17,8 +17,8 @@
 #include <xeus/xinterpreter.hpp>
 #include <xeus/xkernel.hpp>
 #include <xeus/xkernel_configuration.hpp>
-#include <xeus-zmq/xserver_zmq.hpp>
-#include "xeus-zmq/xserver_shell_main.hpp"
+#include <xeus-zmq/xzmq_context.hpp>
+#include <xeus-zmq/xserver_zmq_split.hpp>
 
 #include <nlohmann/json.hpp>
 
@@ -73,10 +73,11 @@ namespace LCompilers::LFortran {
     class custom_interpreter : public xeus::xinterpreter
     {
     private:
+        CompilerOptions compiler_options;
         FortranEvaluator e;
 
     public:
-        custom_interpreter() : e{CompilerOptions()} {
+        custom_interpreter() : compiler_options{}, e{compiler_options} {
             e.compiler_options.interactive = true;
         }
         virtual ~custom_interpreter() = default;
@@ -85,12 +86,13 @@ namespace LCompilers::LFortran {
 
         void configure_impl() override;
 
-        nl::json execute_request_impl(int execution_counter,
-                                      const std::string& code,
-                                      bool silent,
-                                      bool store_history,
-                                      nl::json user_expressions,
-                                      bool allow_stdin) override;
+        void execute_request_impl(send_reply_callback cb,
+                                  int execution_counter,
+                                  const std::string& code,
+                                  //bool silent,
+                                  //bool store_history,
+                                  xeus::execute_request_config config,
+                                  nl::json user_expressions) override;
 
         nl::json complete_request_impl(const std::string& code,
                                        int cursor_pos) override;
@@ -106,13 +108,12 @@ namespace LCompilers::LFortran {
         void shutdown_request_impl() override;
     };
 
-
-    nl::json custom_interpreter::execute_request_impl(int execution_counter, // Typically the cell number
-                                                      const std::string& code, // Code to execute
-                                                      bool /*silent*/,
-                                                      bool /*store_history*/,
-                                                      nl::json /*user_expressions*/,
-                                                      bool /*allow_stdin*/)
+    
+    void custom_interpreter::execute_request_impl(send_reply_callback cb,
+                                                  int execution_counter, // Typically the cell number
+                                                  const std::string& code, // Code to execute
+                                                  xeus::execute_request_config, //config
+                                                  nl::json /*user_expressions*/)
     {
         FortranEvaluator::EvalResult r;
         std::string std_out;
@@ -146,7 +147,8 @@ namespace LCompilers::LFortran {
                     result["evalue"] = msg;
                     result["traceback"] = nl::json::array();
                 }
-                return result;
+                cb(result);
+                return;
             }
             if (startswith(code, "%%showasr")) {
                 code0 = code.substr(code.find("\n")+1);
@@ -175,7 +177,8 @@ namespace LCompilers::LFortran {
                     result["evalue"] = msg;
                     result["traceback"] = nl::json::array();
                 }
-                return result;
+                cb(result);
+                return;
             }
             if (startswith(code, "%%showllvm")) {
                 code0 = code.substr(code.find("\n")+1);
@@ -206,7 +209,8 @@ namespace LCompilers::LFortran {
                     result["evalue"] = msg;
                     result["traceback"] = nl::json::array();
                 }
-                return result;
+                cb(result);
+                return;
             }
             if (startswith(code, "%%showasm")) {
                 code0 = code.substr(code.find("\n")+1);
@@ -237,7 +241,8 @@ namespace LCompilers::LFortran {
                     result["evalue"] = msg;
                     result["traceback"] = nl::json::array();
                 }
-                return result;
+                cb(result);
+                return;
             }
             if (startswith(code, "%%showcpp")) {
                 code0 = code.substr(code.find("\n")+1);
@@ -266,7 +271,8 @@ namespace LCompilers::LFortran {
                     result["evalue"] = msg;
                     result["traceback"] = nl::json::array();
                 }
-                return result;
+                cb(result);
+                return;
             }
             if (startswith(code, "%%showfmt")) {
                 code0 = code.substr(code.find("\n")+1);
@@ -295,7 +301,8 @@ namespace LCompilers::LFortran {
                     result["evalue"] = msg;
                     result["traceback"] = nl::json::array();
                 }
-                return result;
+                cb(result);
+                return;
             }
 
             RedirectStdout s(std_out);
@@ -323,7 +330,8 @@ namespace LCompilers::LFortran {
                 result["ename"] = "CompilerError";
                 result["evalue"] = msg;
                 result["traceback"] = nl::json::array();
-                return result;
+                cb(result);
+                return;
             }
         } catch (const LCompilersException &e) {
             publish_stream("stderr", "LFortran Exception: " + e.msg());
@@ -332,7 +340,8 @@ namespace LCompilers::LFortran {
             result["ename"] = "LCompilersException";
             result["evalue"] = e.msg();
             result["traceback"] = nl::json::array();
-            return result;
+            cb(result);
+            return;
         }
 
         if (std_out.size() > 0) {
@@ -389,9 +398,10 @@ namespace LCompilers::LFortran {
         result["status"] = "ok";
         result["payload"] = nl::json::array();
         result["user_expressions"] = nl::json::object();
-        return result;
+        cb(result);
+        return;
     }
-
+    
     void custom_interpreter::configure_impl()
     {
         // Perform some operations
@@ -482,9 +492,7 @@ namespace LCompilers::LFortran {
 
     int run_kernel(const std::string &connection_filename)
     {
-        using context_type = xeus::xcontext_impl<zmq::context_t>;
-        using context_ptr = std::unique_ptr<context_type>;
-        context_ptr context = context_ptr(new context_type());
+        std::unique_ptr<xeus::xcontext> context = xeus::make_zmq_context();
 
         // Create interpreter instance
         using interpreter_ptr = std::unique_ptr<custom_interpreter>;
