@@ -4,8 +4,9 @@ all whitespace.  It uses a hand written recursive descent parser to figure out
 how to properly tokenize the input. It returns a list of tokens that are then
 fed into our Bison parser, that is shared with the free-form parser.
 
-Note: The prescanner removes CR, so we only handle LF here.
+Note: The prescanner removes CR from regular code, but not from string literals.
 */
+#include <algorithm>
 #include <unordered_map>
 #include <utility>
 
@@ -334,7 +335,10 @@ struct FixedFormRecursiveDescent {
         loc.last = loc_first;
         // auto next=cur; next_line(next);
         //std::cout << "error line " << tostr(cur,next-1) << std::endl;
-        throw LFortran::parser_local::TokenizerError(text, loc);
+        diag.add(diag::Diagnostic(
+            text,
+            diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+        throw parser_local::TokenizerAbort();
     }
 
     // Are the next characters in the `cur` stream equal to `str`?
@@ -430,6 +434,10 @@ struct FixedFormRecursiveDescent {
 
     void next_line(unsigned char *&cur) {
         while (*cur != '\n' && *cur != '\0') {
+            if (*cur == '\r') {
+                cur++;
+                continue;
+            }
             cur++;
         }
         if (*cur == '\n') cur++;
@@ -620,6 +628,10 @@ struct FixedFormRecursiveDescent {
                 // String is ended by delim
                 return true;
             }
+            if (*cur == '\r') {
+                cur++;
+                continue;
+            }
             cur++;
         }
     }
@@ -737,6 +749,7 @@ struct FixedFormRecursiveDescent {
                     y2.int_suffix.int_kind);
             } else if (token == yytokentype::TK_STRING) {
                 std::string tk{tostr(t.tok+1, t.tok + len-1)};
+                tk.erase(std::remove(tk.begin(), tk.end(), '\r'), tk.end());
                 y2.string.from_str(m_a, tk);
             } else {
                 std::string tk{tostr(t.tok, t.tok + len)};
@@ -768,13 +781,19 @@ struct FixedFormRecursiveDescent {
                 Location loc;
                 loc.first = end - string_start;
                 loc.last = end - string_start;
-                throw parser_local::TokenizerError("Expected `)` here to end the condition expression of the if statement ", loc);
+                diag.add(diag::Diagnostic(
+                    "Expected `)` here to end the condition expression of the if statement ",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                throw parser_local::TokenizerAbort();
             }
         } else {
             Location loc;
             loc.first = cur - string_start;
             loc.last = cur - string_start;
-            throw parser_local::TokenizerError("Expected expression after `if`", loc);
+            diag.add(diag::Diagnostic(
+                "Expected expression after `if`",
+                diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+            throw parser_local::TokenizerAbort();
         }
         tokenize_until(end);
         cur = end;
@@ -877,7 +896,7 @@ struct FixedFormRecursiveDescent {
                     cur += io_str.size();
                     unsigned char *start;
                     Location loc;
-                    lex_format(cur, loc, start, diag, false);
+                    lex_format(cur, loc, start, diag, false, this->string_start);
                     locations.push_back(loc);
                     YYSTYPE yylval;
                     yylval.string.p = (char*) start;
@@ -1013,7 +1032,7 @@ struct FixedFormRecursiveDescent {
         }
     }
 
-    bool lex_body_statement(unsigned char *&cur) {
+    bool lex_body_statement(unsigned char *&cur, bool continue_compilation = false) {
         int64_t l = eat_label(cur);
         // handle derived type tokenization
         // this needs to be done before 'lex_declaration'
@@ -1080,6 +1099,24 @@ struct FixedFormRecursiveDescent {
             return true;
         }
 
+        if (next_is(cur, "backspace")) {
+            push_token_advance(cur, "backspace");
+            tokenize_line(cur);
+            return true;
+        }
+
+        if (next_is(cur, "rewind")) {
+            push_token_advance(cur, "rewind");
+            tokenize_line(cur);
+            return true;
+        }
+
+        if (next_is(cur, "endfile")) {
+            push_token_advance(cur, "endfile");
+            tokenize_line(cur);
+            return true;
+        }
+
         if (next_is(cur, "interface")) {
             lex_interface(cur);
             return true;
@@ -1186,6 +1223,13 @@ struct FixedFormRecursiveDescent {
             // Undo the label, as it will be handled later
             undo_label(cur);
         }
+        if (next_is(cur, "endprogram") || next_is(cur, "end") || next_is(cur, "contains") || next_is(cur, "subroutine") || next_is(cur, "function")) {
+            return false;
+        }
+        if (continue_compilation) {
+            tokenize_line(cur);
+            return true;
+        }
 
         return false;
     }
@@ -1201,13 +1245,19 @@ struct FixedFormRecursiveDescent {
                 Location loc;
                 loc.first = cur-string_start;
                 loc.last = cur-string_start;
-                throw parser_local::TokenizerError("Expected 'to' here", loc);
+                diag.add(diag::Diagnostic(
+                    "Expected 'to' here",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                throw parser_local::TokenizerAbort();
             }
         } else {
             Location loc;
             loc.first = cur-string_start;
             loc.last = cur-string_start;
-            throw parser_local::TokenizerError("Expected integer after `assign`", loc);
+            diag.add(diag::Diagnostic(
+                "Expected integer after `assign`",
+                diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+            throw parser_local::TokenizerAbort();
         }
     }
 
@@ -1236,7 +1286,10 @@ struct FixedFormRecursiveDescent {
             Location loc;
             loc.first = 1;
             loc.last = 1;
-            throw parser_local::TokenizerError("End of file encountered in labeled do loop (loop is not terminated)", loc);
+            diag.add(diag::Diagnostic(
+                "End of file encountered in labeled do loop (loop is not terminated)",
+                diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+            throw parser_local::TokenizerAbort();
         }
         int64_t label = eat_label(cur);
         if (label != do_label) {
@@ -1374,8 +1427,10 @@ struct FixedFormRecursiveDescent {
                 Location loc;
                 loc.first = cur - string_start;
                 loc.last = cur - string_start;
-                throw parser_local::TokenizerError("Expected an executable "
-                    "statement inside do concurrent loop", loc);
+                diag.add(diag::Diagnostic(
+                    "Expected an executable statement inside do concurrent loop",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                throw parser_local::TokenizerAbort();
             }
         }
     }
@@ -1423,7 +1478,10 @@ struct FixedFormRecursiveDescent {
                 Location loc;
                 loc.first = cur-string_start;
                 loc.last = cur-string_start;
-                throw parser_local::TokenizerError("Expected an executable statement inside a do loop", loc);
+                diag.add(diag::Diagnostic(
+                    "Expected an executable statement inside a do loop",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                throw parser_local::TokenizerAbort();
             };
         }
     }
@@ -1438,7 +1496,10 @@ struct FixedFormRecursiveDescent {
                 Location loc;
                 loc.first = cur-string_start;
                 loc.last = cur-string_start;
-                throw parser_local::TokenizerError("Expected an executable statement inside a labeled do loop", loc);
+                diag.add(diag::Diagnostic(
+                    "Expected an executable statement inside a labeled do loop",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                throw parser_local::TokenizerAbort();
             };
             if (!label_last(do_label)) {
                 // The lex_body_statement() above was a labeled loop that ended
@@ -1515,6 +1576,12 @@ struct FixedFormRecursiveDescent {
             push_token_no_advance(cur, "continue");
             push_token_no_advance(cur, "\n");
         }
+        if (next_is(cur, "contains")) {
+            push_token_advance(cur, "contains");
+            push_token_no_advance(cur, "\n");
+            next_line(cur);
+            while(lex_procedure(cur));
+        }
         if (next_is(cur, "endsubroutine")) {
             push_token_advance(cur, "endsubroutine");
             tokenize_line(cur);
@@ -1541,18 +1608,40 @@ struct FixedFormRecursiveDescent {
                    end
             ```
     */
-    void lex_program(unsigned char *&cur, bool explicit_program) {
+    void lex_program(unsigned char *&cur, bool explicit_program, bool continue_compilation, diag::Diagnostics &diagnostics) {
         if (explicit_program) {
             push_token_advance(cur, "program");
             tokenize_line(cur);
         }
-        while(lex_body_statement(cur));
+        while(lex_body_statement(cur, continue_compilation));
         eat_label(cur);
         if (next_is(cur, "contains")) {
             push_token_advance(cur, "contains");
             push_token_no_advance(cur, "\n");
             next_line(cur); // Does not generate any code?
             while(lex_procedure(cur));
+        } else if (next_is(cur, "subroutine") || next_is(cur, "function")) {
+            Location loc;
+            token_loc(cur, cur + 1, loc);
+            if (continue_compilation) {
+                diagnostics.add(diag::Diagnostic(
+                    "Expecting contains keyword before procedure definition",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                if (next_is(cur, "subroutine")) {
+                    while (!next_is(cur, "endprogram")) {
+                        next_line(cur);
+                    }
+                } else if (next_is(cur, "function")) {
+                    while (!next_is(cur, "endprogram")) {
+                        next_line(cur);
+                    }
+                }
+            } else {
+                diagnostics.add(diag::Diagnostic(
+                    "Expecting contains keyword before procedure definition",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                throw parser_local::TokenizerAbort();
+            }
         }
         if (next_is(cur, "endprogram")) {
             push_token_advance(cur, "endprogram");
@@ -1725,9 +1814,9 @@ struct FixedFormRecursiveDescent {
     }
 
     bool lex_procedure(unsigned char *&cur) {
-	const std::vector<std::string> subroutine_keywords{"recursive", "pure",
+	const std::vector<std::string> subroutine_keywords{"recursive", "non_recursive", "pure",
             "elemental"};
-        const std::vector<std::string> function_keywords{"recursive", "pure",
+        const std::vector<std::string> function_keywords{"recursive", "non_recursive", "pure",
             "elemental", "real*", "real",
 	    "character*(*)",
             "character*", "character",
@@ -1746,7 +1835,7 @@ struct FixedFormRecursiveDescent {
         }
     }
 
-    void lex_global_scope_item(unsigned char *&cur) {
+    void lex_global_scope_item(unsigned char *&cur, bool continue_compilation, diag::Diagnostics &diagnostics) {
         // we can define a global assignment
         unsigned char *nline = cur; next_line(nline);
         // eat_label(cur);
@@ -1755,7 +1844,7 @@ struct FixedFormRecursiveDescent {
             tokenize_line(cur);
         }
         if (is_program(cur)) {
-            lex_program(cur, true);
+            lex_program(cur, true, continue_compilation, diagnostics);
         } else if (next_is(cur, "interface")) {
             lex_interface(cur);
         } else if (is_module(cur)) {
@@ -1769,18 +1858,18 @@ struct FixedFormRecursiveDescent {
             push_token_no_advance(cur, "program");
             push_token_no_advance_token(cur, "implicit_program_lfortran", TK_NAME);
             push_token_no_advance(cur, "\n");
-            lex_program(cur, false);
+            lex_program(cur, false, continue_compilation, diagnostics);
         } else {
             error(cur, "ICE: Cannot recognize global scope entity");
         }
     }
 
-    void lex_global_scope(unsigned char *&cur) {
+    void lex_global_scope(unsigned char *&cur, bool continue_compilation, diag::Diagnostics &diagnostics) {
         auto next = cur;
         while (*cur != '\0') {
             // eat_label(cur);
             next_line(next);
-            lex_global_scope_item(cur);
+            lex_global_scope_item(cur, continue_compilation, diagnostics);
             next = cur;
         }
         push_token_no_advance(cur, "EOF");
@@ -1788,7 +1877,7 @@ struct FixedFormRecursiveDescent {
 
 };
 
-bool FixedFormTokenizer::tokenize_input(diag::Diagnostics &diagnostics, Allocator &al) {
+bool FixedFormTokenizer::tokenize_input(diag::Diagnostics &diagnostics, Allocator &al, bool continue_compilation) {
     // We use a recursive descent parser.  We are starting at the global scope
     try {
         FixedFormRecursiveDescent f(diagnostics, al);
@@ -1798,15 +1887,14 @@ bool FixedFormTokenizer::tokenize_input(diag::Diagnostics &diagnostics, Allocato
         f.t.string_start = string_start;
         f.t.cur_line = string_start;
         f.t.line_num = 1;
-        f.lex_global_scope(cur);
+        f.lex_global_scope(cur, continue_compilation, diagnostics);
         tokens = std::move(f.tokens);
         stypes = std::move(f.stypes);
         locations = std::move(f.locations);
         LCOMPILERS_ASSERT(tokens.size() == stypes.size())
         LCOMPILERS_ASSERT(tokens.size() == locations.size())
         tokenized = true;
-    } catch (const parser_local::TokenizerError &e) {
-        diagnostics.diagnostics.push_back(e.d);
+    } catch (const parser_local::TokenizerAbort &) {
         return false;
     }
     return true;
