@@ -389,7 +389,7 @@ end function)";
     co.platform = LCompilers::get_platform();
     LCompilers::Result<std::unique_ptr<LCompilers::LLVMModule>>
         res = LCompilers::asr_to_llvm(*asr, diagnostics, e.get_context(), al,
-            lpm, co, "f", "", "");
+            lpm, co, "f", "", "", lm);
     REQUIRE(res.ok);
     std::unique_ptr<LCompilers::LLVMModule> m = std::move(res.result);
     //std::cout << "Module:" << std::endl;
@@ -427,7 +427,7 @@ end function)";
     lpm.use_default_passes();
     LCompilers::Result<std::unique_ptr<LCompilers::LLVMModule>>
         res = LCompilers::asr_to_llvm(*asr, diagnostics, e.get_context(), al,
-            lpm, compiler_options, "f", "", "");
+            lpm, compiler_options, "f", "", "", lm);
     REQUIRE(res.ok);
     std::unique_ptr<LCompilers::LLVMModule> m = std::move(res.result);
     //std::cout << "Module:" << std::endl;
@@ -625,12 +625,12 @@ end module funcmod)");
     r = e.evaluate2("add(2.0, 5.0)");
     CHECK(r.ok);
     CHECK(r.result.type == FortranEvaluator::EvalResult::real4);
-    CHECK(abs(r.result.f32 - 7.0) <= 1e-8 );
+    CHECK(std::abs(r.result.f32 - 7.0) <= 1e-8 );
 
     r = e.evaluate2("subtract(2.0, 5.0)");
     CHECK(r.ok);
     CHECK(r.result.type == FortranEvaluator::EvalResult::real4);
-    CHECK(abs(r.result.f32 - (-3.0)) <= 1e-8 );
+    CHECK(std::abs(r.result.f32 - (-3.0)) <= 1e-8 );
 }
 
 // Tests passing the complex struct by reference
