@@ -393,6 +393,10 @@ class ASRBuilder {
         return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::ComplexToInteger, t, nullptr));
     }
 
+    inline ASR::expr_t* c2c_t(ASR::expr_t* x, ASR::ttype_t* t) {
+        return EXPR(ASR::make_Cast_t(al, loc, x, ASR::cast_kindType::ComplexToComplex, t, nullptr));
+    }
+
     inline ASR::expr_t* i2r_t(ASR::expr_t* x, ASR::ttype_t* t) {
         ASR::expr_t* value = ASRUtils::expr_value(x);
         if ( value != nullptr ) {
@@ -954,9 +958,7 @@ class ASRBuilder {
         idx_vars, al)
 
     ASR::expr_t *ArrayConstant(std::vector<ASR::expr_t *> elements,
-            ASR::ttype_t *base_type, bool cast2descriptor=true) {
-        // This function only creates array with rank one
-        // TODO: Support other dimensions
+            ASR::ttype_t *base_type, bool cast2descriptor=true, ASR::ttype_t *array_type = nullptr) {
         Vec<ASR::expr_t *> m_eles; m_eles.reserve(al, 1);
         for (auto &x: elements) m_eles.push_back(al, x);
 
@@ -964,6 +966,11 @@ class ASRBuilder {
         ASR::expr_t *arr_constant = EXPR(ASRUtils::make_ArrayConstructor_t_util(al, loc,
             m_eles.p, m_eles.n, fixed_size_type, ASR::arraystorageType::ColMajor));
 
+        // Set multi-dimensional type
+        if (array_type) {
+            ASR::down_cast<ASR::ArrayConstant_t>(arr_constant)->m_type = array_type;
+        }
+
         if (cast2descriptor) {
             return cast_to_descriptor(al, arr_constant);
         } else {
@@ -1018,16 +1025,17 @@ class ASRBuilder {
         }
     }
 
-    ASR::stmt_t *Allocate(ASR::expr_t *m_a, Vec<ASR::dimension_t> dims) {
+    ASR::stmt_t *Allocate(ASR::expr_t *m_a, Vec<ASR::dimension_t> dims, ASR::symbol_t* sym_subclass = nullptr) {
         Vec<ASR::alloc_arg_t> alloc_args; alloc_args.reserve(al, 1);
         ASR::alloc_arg_t alloc_arg;
         alloc_arg.loc = loc;
         alloc_arg.m_a = m_a;
         alloc_arg.m_dims = dims.p;
         alloc_arg.n_dims = dims.n;
-        alloc_arg.m_type = nullptr;
+        alloc_arg.m_type = sym_subclass ? ASRUtils::type_get_past_allocatable_pointer(
+            ASRUtils::symbol_type(sym_subclass)) : nullptr;
         alloc_arg.m_len_expr = nullptr;
-        alloc_arg.m_sym_subclass = nullptr;
+        alloc_arg.m_sym_subclass = sym_subclass;
         alloc_args.push_back(al, alloc_arg);
         return STMT(ASR::make_Allocate_t(al, loc, alloc_args.p, 1,
             nullptr, nullptr, nullptr));
