@@ -11,11 +11,10 @@ namespace LCompilers::CastingUtil {
     int get_src_dest(ASR::expr_t* left_expr, ASR::expr_t* right_expr,
                       ASR::expr_t*& src_expr, ASR::expr_t*& dest_expr,
                       ASR::ttype_t*& src_type, ASR::ttype_t*& dest_type,
-                      bool is_assign);
+                      bool is_assign, bool allow_int_to_float=false);
 
-    ASR::expr_t* perform_casting(ASR::expr_t* expr, ASR::ttype_t* src,
-                                 ASR::ttype_t* dest, Allocator& al,
-                                 const Location& loc);
+    ASR::expr_t* perform_casting(ASR::expr_t* expr, ASR::ttype_t* dest,
+                                Allocator& al, const Location& loc);
 }
 
 #endif // LFORTRAN_CASTING_UTILS_H
