@@ -943,9 +943,12 @@ public:
         visit_expr(*x.m_target);
         r += src;
         r += ", RHS = ";
-        visit_expr(*x.m_value);
-        r += src;
-        handle_line_truncation(r, 2);
+        for (size_t i = 0; i < x.n_components; i++) {
+            visit_expr(*x.m_components[i]);
+            r += src;
+            r += ", ";
+            handle_line_truncation(r, 2);
+        }
         r += "\n";
         src = r;
     }
@@ -1634,6 +1637,28 @@ public:
         src = out;
     }
 
+    void handle_cmplx_intrinsic(std::string &out, const ASR::IntrinsicElementalFunction_t &x) {
+        visit_expr(*x.m_args[0]);
+        std::string arg1 = src;
+
+        bool first_is_complex = ASR::is_a<ASR::Complex_t>(*ASRUtils::expr_type(x.m_args[0]));
+
+        out += "(" + arg1;
+
+        if (!first_is_complex && x.n_args >= 2) {
+            visit_expr(*x.m_args[1]);
+            out += ", " + src;
+        }
+
+        if (x.n_args == 3) {
+            visit_expr(*x.m_args[2]);
+            out += ", kind=" + src;
+        }
+
+        out += ")";
+        src = out;
+    }
+
     void visit_IntrinsicElementalFunction_helper(std::string &out, std::string func_name, const ASR::IntrinsicElementalFunction_t &x) {
         if ( x.m_intrinsic_id == static_cast<int64_t>(ASRUtils::IntrinsicElementalFunctions::CompilerVersion) ) {
             src = "";
@@ -1644,6 +1669,10 @@ public:
         }
         src = "";
         out += func_name;
+        if (to_lower(func_name) == "cmplx") {
+            handle_cmplx_intrinsic(out, x);
+            return;
+        }
         if (x.n_args > 0) visit_expr(*x.m_args[0]);
         out += "(" + src;
         for (size_t i = 1; i < x.n_args; i++) {
@@ -2118,6 +2147,11 @@ public:
         src = r;
     }
 
+    void visit_ArrayRank(const ASR::ArrayRank_t &x) {
+        visit_expr(*x.m_v);
+        src = "rank(" + src + ")";
+    }
+
     void visit_ArrayBound(const ASR::ArrayBound_t &x) {
         std::string r = "";
         if (x.m_bound == ASR::arrayboundType::UBound) {
