@@ -3,6 +3,7 @@
 #include <libasr/codegen/llvm_utils.h>
 #include <libasr/asr_utils.h>
 #include "llvm_array_utils.h"
+#include <libasr/codegen/llvm_compat.h>
 
 namespace LCompilers {
 
@@ -18,7 +19,7 @@ namespace LCompilers {
                             llvm::Type::getInt64Ty(context)
                         }, false);
                 fn = llvm::Function::Create(function_type,
-                        llvm::Function::ExternalLinkage, func_name, module);
+                        llvm::Function::ExternalLinkage, func_name, &module);
             }
             arg_size = builder.CreateSExt(arg_size, llvm::Type::getInt64Ty(context));
             std::vector<llvm::Value*> args = {arg_size};
@@ -36,7 +37,7 @@ namespace LCompilers {
                             llvm::Type::getInt64Ty(context)
                         }, false);
                 fn = llvm::Function::Create(function_type,
-                        llvm::Function::ExternalLinkage, func_name, module);
+                        llvm::Function::ExternalLinkage, func_name, &module);
             }
             std::vector<llvm::Value*> args = {
                 builder.CreateBitCast(ptr, llvm::Type::getInt8Ty(context)->getPointerTo()),
@@ -130,9 +131,9 @@ namespace LCompilers {
                 nullptr, ASRUtils::get_contained_type(asr_arg_type), llvm_utils->module);
             llvm::StructType* tmp_struct_type = tkr2array[asr_arg_type_code].first;
             if( tmp_struct_type->getElementType(0)->isArrayTy() ) {
-                first_ele_ptr = llvm_utils->create_gep2(element_type, get_pointer_to_data(tmp), 0);
+                first_ele_ptr = llvm_utils->create_gep2(element_type, get_pointer_to_data(arg_type, tmp), 0);
             } else if( tmp_struct_type->getNumElements() < 5 ) {
-                first_ele_ptr = llvm_utils->CreateLoad2(element_type, get_pointer_to_data(tmp));
+                first_ele_ptr = llvm_utils->CreateLoad2(element_type, get_pointer_to_data(arg_type, tmp));
             } else if( tmp_struct_type->getNumElements() == 5 ) {
                 return tmp;
             }
@@ -299,7 +300,7 @@ namespace LCompilers {
 
             llvm::Value* llvm_size = llvm_utils->CreateAlloca(*builder, llvm::Type::getInt32Ty(context));
             builder->CreateStore(prod, llvm_size);
-            llvm::Value* first_ptr = get_pointer_to_data(arr);
+            llvm::Value* first_ptr = get_pointer_to_data(arr_ty, arr);
             llvm::Value* arr_first = nullptr;
 
             if( !co.stack_arrays ) {
@@ -317,9 +318,6 @@ namespace LCompilers {
                 arr_first = llvm_utils->CreateAlloca(*builder,
                     llvm_data_type, llvm_utils->CreateLoad2(llvm::Type::getInt32Ty(context), llvm_size));
             }
-#if LLVM_VERSION_MAJOR > 16
-            llvm_utils->ptr_type_deprecated[arr_first] = llvm_data_type;
-#endif
             builder->CreateStore(arr_first, first_ptr);
         }
 
@@ -354,9 +352,6 @@ namespace LCompilers {
             std::vector<std::pair<llvm::Value*, llvm::Value*>>& llvm_dims, llvm::Value* string_len,
             llvm::Module* module, bool realloc) {
             arr = llvm_utils->CreateLoad2(arr_type->getPointerTo(), arr);
-#if LLVM_VERSION_MAJOR > 16
-            llvm_utils->ptr_type_deprecated[arr] = arr_type;
-#endif
             llvm::Value* offset_val = llvm_utils->create_gep2(arr_type, arr, 1);
             builder->CreateStore(llvm::ConstantInt::get(context, llvm::APInt(32, 0)),
                                     offset_val);
@@ -375,7 +370,7 @@ namespace LCompilers {
                 builder->CreateStore(dim_size, dim_size_ptr);
                 prod = builder->CreateMul(prod, dim_size);
             }
-            llvm::Value* ptr2firstptr = get_pointer_to_data(arr);
+            llvm::Value* ptr2firstptr = get_pointer_to_data(arr_type,  arr);
             llvm::AllocaInst *arg_size = llvm_utils->CreateAlloca(*builder, llvm::Type::getInt32Ty(context));
 
             // Handle special allocation for strings. All Other types are handled the same.
@@ -403,9 +398,6 @@ namespace LCompilers {
                         *builder, llvm_utils->CreateLoad2(llvm::Type::getInt32Ty(context), arg_size));
                 }
                 llvm::Value* first_ptr = builder->CreateBitCast(ptr_as_char_ptr, ptr_type);
-#if LLVM_VERSION_MAJOR > 16
-                llvm_utils->ptr_type_deprecated[first_ptr] = ptr_type;
-#endif
                 builder->CreateStore(first_ptr, ptr2firstptr);
             }
         }
@@ -501,7 +493,7 @@ namespace LCompilers {
             llvm::Value* target, ASR::ttype_t* target_type, ASR::expr_t* target_expr,
             llvm::Value** lbs, llvm::Value** ubs,
             llvm::Value** ds, llvm::Value** non_sliced_indices,
-            int value_rank, int target_rank) {
+            int value_rank, int target_rank, LocationManager& lm) {
             llvm::Value* value_desc_data = llvm_utils->CreateLoad2(value_el_type->getPointerTo(), get_pointer_to_data(target_expr, ASRUtils::type_get_past_allocatable_pointer(target_type), value_desc, llvm_utils->module));
             std::vector<llvm::Value*> section_first_indices;
             for( int i = 0; i < value_rank; i++ ) {
@@ -516,7 +508,7 @@ namespace LCompilers {
             llvm::Type* type_llvm = llvm_utils->get_type_from_ttype_t_util(
                 target_expr, ASRUtils::type_get_past_allocatable_pointer(target_type), llvm_utils->module);
             llvm::Value* target_offset = cmo_convertor_single_element(
-                type_llvm, value_desc, section_first_indices, value_rank, false);
+                type_llvm, value_desc, section_first_indices, value_rank, false, lm);
 
             if(ASRUtils::is_character(*value_type)){
                 LCOMPILERS_ASSERT_MSG(ASRUtils::is_descriptorString(value_type),
@@ -587,7 +579,7 @@ namespace LCompilers {
             llvm::Value* target, ASR::ttype_t* target_type, ASR::expr_t* target_expr,
             llvm::Value** lbs, llvm::Value** ubs,
             llvm::Value** ds, llvm::Value** non_sliced_indices,
-            llvm::Value** llvm_diminfo, int value_rank, int target_rank) {
+            llvm::Value** llvm_diminfo, int value_rank, int target_rank, LocationManager& lm) {
             std::vector<llvm::Value*> section_first_indices;
             for( int i = 0; i < value_rank; i++ ) {
                 if( ds[i] != nullptr ) {
@@ -599,7 +591,7 @@ namespace LCompilers {
                 }
             }
             llvm::Value* target_offset = cmo_convertor_single_element_data_only(
-                llvm_diminfo, section_first_indices, value_rank, false);
+                llvm_diminfo, section_first_indices, value_rank, false, lm);
             if(ASRUtils::is_character(*value_type)){
                 LCOMPILERS_ASSERT_MSG(ASRUtils::is_descriptorString(value_type),
                     "Only descriptor strings are supported for now");
@@ -670,8 +662,8 @@ namespace LCompilers {
             return llvm_utils->create_ptr_gep2(dim_des, dim_des_arr, dim);
         }
 
-        llvm::Value* SimpleCMODescriptor::get_pointer_to_data(llvm::Value* arr) {
-            return llvm_utils->create_gep_deprecated(arr, 0);
+        llvm::Value* SimpleCMODescriptor::get_pointer_to_data(llvm::Type* type, llvm::Value* arr) {
+            return llvm_utils->create_gep2(type, arr, 0);
         }
 
         llvm::Value* SimpleCMODescriptor::get_pointer_to_data(ASR::expr_t* arr_expr, ASR::ttype_t* arr_type, llvm::Value* arr, llvm::Module* module) {
@@ -718,7 +710,7 @@ namespace LCompilers {
 
         llvm::Value* SimpleCMODescriptor::cmo_convertor_single_element(
             llvm::Type* type, llvm::Value* arr, std::vector<llvm::Value*>& m_args,
-            int n_args, bool check_for_bounds, std::string array_name) {
+            int n_args, bool check_for_bounds, LocationManager& lm, std::string array_name, std::string infile, Location loc) {
             llvm::Value* dim_des_arr_ptr = llvm_utils->CreateLoad2(
                 dim_des->getPointerTo(), llvm_utils->create_gep2(type, arr, 2));
             llvm::Value* idx = llvm::ConstantInt::get(context, llvm::APInt(32, 0));
@@ -742,9 +734,12 @@ namespace LCompilers {
                     llvm_utils->generate_runtime_error(builder->CreateOr(
                                                             lbound_check,
                                                             ubound_check),
-                                            "Runtime error: Array '%s' index out of bounds.\n\n"
+                                                "Runtime error: Array '%s' index out of bounds.\n\n"
                                                      "Tried to access index %d of dimension %d, but valid range is %d to %d.\n",
-                                                     builder->CreateGlobalStringPtr(array_name),
+                                                     infile,
+                                                     loc,
+                                                     lm,
+                                                     LCompilers::create_global_string_ptr(context, *builder->GetInsertBlock()->getParent()->getParent(), *builder, array_name),
                                                      req_idx,
                                                      dimension,
                                                      lval,
@@ -759,7 +754,7 @@ namespace LCompilers {
 
         llvm::Value* SimpleCMODescriptor::cmo_convertor_single_element_data_only(
             llvm::Value** llvm_diminfo, std::vector<llvm::Value*>& m_args,
-            int n_args, bool check_for_bounds, bool is_unbounded_pointer_to_data, std::string array_name) {
+            int n_args, bool check_for_bounds,LocationManager& lm, bool is_unbounded_pointer_to_data, std::string array_name, std::string infile, Location loc) {
             llvm::Value* prod = llvm::ConstantInt::get(context, llvm::APInt(32, 1));
             llvm::Value* idx = llvm::ConstantInt::get(context, llvm::APInt(32, 0));
             for( int r = 0, r1 = 0; r < n_args; r++ ) {
@@ -789,7 +784,10 @@ namespace LCompilers {
                                                                 ubound_check),
                                                 "Runtime error: Array '%s' index out of bounds.\n\n"
                                                         "Tried to access index %d of dimension %d, but valid range is %d to %d.\n",
-                                                        builder->CreateGlobalStringPtr(array_name),
+                                                        infile,
+                                                        loc,
+                                                        lm,
+                                                        LCompilers::create_global_string_ptr(context, *builder->GetInsertBlock()->getParent()->getParent(), *builder, array_name),
                                                         req_idx,
                                                         dimension,
                                                         lval,
@@ -802,14 +800,14 @@ namespace LCompilers {
         }
 
         llvm::Value* SimpleCMODescriptor::get_single_element(llvm::Type *type, llvm::Value* array,
-            std::vector<llvm::Value*>& m_args, int n_args, ASR::ttype_t* asr_type, ASR::expr_t* expr, bool data_only,
+            std::vector<llvm::Value*>& m_args, int n_args, ASR::ttype_t* asr_type, ASR::expr_t* expr, LocationManager& lm, bool data_only,
             bool is_fixed_size, llvm::Value** llvm_diminfo, bool polymorphic,
-            llvm::Type* polymorphic_type, bool is_unbounded_pointer_to_data, bool check_for_bounds, std::string array_name) {
+            llvm::Type* polymorphic_type, bool is_unbounded_pointer_to_data, bool check_for_bounds, std::string array_name, std::string infile) {
             llvm::Value* tmp = nullptr;
             llvm::Value* idx = nullptr;
             if( data_only || is_fixed_size ) {
                 LCOMPILERS_ASSERT(llvm_diminfo);
-                idx = cmo_convertor_single_element_data_only(llvm_diminfo, m_args, n_args, check_for_bounds, is_unbounded_pointer_to_data, array_name);
+                idx = cmo_convertor_single_element_data_only(llvm_diminfo, m_args, n_args, check_for_bounds, lm, is_unbounded_pointer_to_data, array_name, infile, expr->base.loc);
                 if(ASRUtils::is_character(*asr_type)){// Special handling for array of strings.
                     tmp = llvm_utils->get_string_element_in_array(ASR::down_cast<ASR::String_t>(ASRUtils::extract_type(asr_type)), array, idx);
                 } else {
@@ -822,7 +820,7 @@ namespace LCompilers {
             } else {
                 llvm::Type* array_type = llvm_utils->get_type_from_ttype_t_util(
                     expr, ASRUtils::type_get_past_allocatable_pointer(asr_type), llvm_utils->module);
-                idx = cmo_convertor_single_element(array_type, array, m_args, n_args, check_for_bounds, array_name);
+                idx = cmo_convertor_single_element(array_type, array, m_args, n_args, check_for_bounds, lm, array_name, infile, expr->base.loc);
                 llvm::Value* full_array = llvm_utils->CreateLoad2(type->getPointerTo(), get_pointer_to_data(expr, ASRUtils::type_get_past_allocatable_pointer(asr_type), array, llvm_utils->module));
                 if(ASRUtils::is_character(*asr_type)){
                     tmp = llvm_utils->get_string_element_in_array(ASR::down_cast<ASR::String_t>(ASRUtils::extract_type(asr_type)), full_array, idx);
@@ -844,6 +842,21 @@ namespace LCompilers {
             llvm::Value* memory_holder{}; // ptr_ptr_to_data
             llvm::PointerType* memory_holder_type;
             ASR::ttype_t* array_type = ASRUtils::expr_type(array_exp);
+            // Check if array descriptor pointer is NULL
+            // Example :: For allocatable members in derived types
+            // If the descriptor pointer itself is NULL, the array is not allocated
+            llvm::BasicBlock *entryBB = builder->GetInsertBlock();
+            llvm::Value* array_ptr_int = builder->CreatePtrToInt(array, llvm::Type::getInt64Ty(context));
+            llvm::Value* is_desc_null = builder->CreateICmpEQ(array_ptr_int, 
+                llvm::ConstantInt::get(llvm::Type::getInt64Ty(context), 0));
+            // Create blocks for conditional execution
+            llvm::Function *fn = entryBB->getParent();
+            llvm::BasicBlock *checkDataBB = llvm::BasicBlock::Create(context, "check_data", fn);
+            llvm::BasicBlock *mergeBB = llvm::BasicBlock::Create(context, "merge_allocated", fn);
+            // If descriptor is NULL, skip to merge with false result
+            builder->CreateCondBr(is_desc_null, mergeBB, checkDataBB);
+            // Check data pointer block
+            builder->SetInsertPoint(checkDataBB);
             // Handle an array of strings
             memory_holder_type = ASRUtils::is_character(*array_type) ?
                 llvm_utils->character_type
@@ -854,25 +867,32 @@ namespace LCompilers {
                     array_exp,
                     ASRUtils::extract_type(array_type), llvm_utils->module)->getPointerTo();
                 ASR::String_t* str = ASRUtils::get_string_type(array_type);
+
                 memory_holder = llvm_utils->get_string_data(
                         str,
-                        builder->CreateLoad(load_type, get_pointer_to_data(array)), true);
+                        builder->CreateLoad(load_type, get_pointer_to_data(array_exp, array_type, array, llvm_utils->module)), true);
             } else {
                 memory_holder = get_pointer_to_data(array_exp, array_type, array, llvm_utils->module);
             }
-
-            return builder->CreateICmpNE(
+            llvm::Value* is_data_allocated = builder->CreateICmpNE(
                 builder->CreatePtrToInt(llvm_utils->CreateLoad2(memory_holder_type, memory_holder),
                     llvm::Type::getInt64Ty(context)),
                 builder->CreatePtrToInt(llvm::ConstantPointerNull::get(memory_holder_type),
                     llvm::Type::getInt64Ty(context)));
+            builder->CreateBr(mergeBB);
+            // Merge block with PHI node
+            builder->SetInsertPoint(mergeBB);
+            llvm::PHINode *result = builder->CreatePHI(llvm::Type::getInt1Ty(context), 2, "is_allocated");
+            result->addIncoming(llvm::ConstantInt::getFalse(context), entryBB); // from entry (NULL case)
+            result->addIncoming(is_data_allocated, checkDataBB); // from check_data block
+            return result;
         }
 
-        void SimpleCMODescriptor::reset_is_allocated_flag(llvm::Value* array,
+        void SimpleCMODescriptor::reset_is_allocated_flag(llvm::Type* typ_tmp, llvm::Value* array,
             llvm::Type* llvm_data_type) {
             builder->CreateStore(
                 llvm::ConstantPointerNull::get(llvm_data_type->getPointerTo()),
-                get_pointer_to_data(array)
+                get_pointer_to_data(typ_tmp, array)
             );
         }
 
@@ -927,11 +947,11 @@ namespace LCompilers {
             // Deep copy data from array to reshaped.
             llvm::Value* num_elements = this->get_array_size(arr_type, array, nullptr, 4);
 
-            llvm::Value* first_ptr = this->get_pointer_to_data(reshaped);
+            llvm::Value* first_ptr = this->get_pointer_to_data(arr_type, reshaped);
             llvm::Value* arr_first = llvm_utils->CreateAlloca(*builder, llvm_data_type, num_elements);
             builder->CreateStore(arr_first, first_ptr);
 
-            llvm::Value* ptr2firstptr = this->get_pointer_to_data(array);
+            llvm::Value* ptr2firstptr = this->get_pointer_to_data(arr_type, array);
             llvm::DataLayout data_layout(module->getDataLayout());
             uint64_t size = data_layout.getTypeAllocSize(llvm_data_type);
             llvm::Value* llvm_size = llvm::ConstantInt::get(context, llvm::APInt(32, size));
@@ -949,7 +969,7 @@ namespace LCompilers {
                 builder->CreateStore(llvm_utils->CreateLoad2(i32, llvm_utils->create_gep2(arr_type, array, 1)),
                             llvm_utils->create_gep2(arr_type, reshaped, 1));
                 llvm::Value* n_dims = this->get_array_size(shape_type, shape, nullptr, 4);
-                llvm::Value* shape_data = llvm_utils->CreateLoad2(i32->getPointerTo(), this->get_pointer_to_data(shape));
+                llvm::Value* shape_data = llvm_utils->CreateLoad2(i32->getPointerTo(), this->get_pointer_to_data(shape_type, shape));
                 llvm::Value* dim_des_val = llvm_utils->create_gep2(arr_type, reshaped, 2);
                 llvm::Value* dim_des_first = llvm_utils->CreateAlloca(*builder, dim_des, n_dims);
                 builder->CreateStore(n_dims, this->get_rank(arr_type, reshaped, true));
@@ -995,7 +1015,7 @@ namespace LCompilers {
             llvm::Module* module, ASR::ttype_t* asr_data_type, bool reserve_memory) {
             llvm::Value* num_elements = this->get_array_size(src_ty, src, nullptr, 4);
 
-            llvm::Value* first_ptr = this->get_pointer_to_data(dest);
+            llvm::Value* first_ptr = this->get_pointer_to_data(dest_ty, dest);
             llvm::Type* llvm_data_type = tkr2array[ASRUtils::get_type_code(ASRUtils::type_get_past_pointer(
                 ASRUtils::type_get_past_allocatable(asr_data_type)), false, false)].second;
             if( reserve_memory ) {
@@ -1003,7 +1023,7 @@ namespace LCompilers {
                 builder->CreateStore(arr_first, first_ptr);
             }
 
-            llvm::Value* ptr2firstptr = this->get_pointer_to_data(src);
+            llvm::Value* ptr2firstptr = this->get_pointer_to_data(src_ty, src);
             llvm::DataLayout data_layout(module->getDataLayout());
             uint64_t size = data_layout.getTypeAllocSize(llvm_data_type);
             llvm::Value* llvm_size = llvm::ConstantInt::get(context, llvm::APInt(32, size));
@@ -1055,13 +1075,26 @@ namespace LCompilers {
         void SimpleCMODescriptor::copy_array_move_allocation(llvm::Type* src_ty, llvm::Value* src, llvm::Type* dest_ty, llvm::Value* dest,
             llvm::Module* module, ASR::expr_t* array_exp, ASR::ttype_t* asr_data_type) {
 
-            llvm::Value* first_ptr = this->get_pointer_to_data(dest);
-            llvm::Value* src_data_ptr = this->get_pointer_to_data(src);
+            llvm::Value* first_ptr = this->get_pointer_to_data(dest_ty, dest);
+            llvm::Value* src_data_ptr = this->get_pointer_to_data(src_ty, src);
             llvm::Type* llvm_data_type =  llvm_utils->get_el_type(array_exp, ASRUtils::extract_type(asr_data_type), module);
-            builder->CreateStore(builder->CreateLoad(llvm_data_type->getPointerTo(), src_data_ptr), first_ptr);
+            if(ASRUtils::is_character(*asr_data_type)){ // `{ %string_descriptor*, i32, %dimension_descriptor*, i1, i32 }` -- Copy The whole string descriptor.
+                llvm::Value* dest_str_desc_loaded {}; // %string_descriptor*
+                llvm::Value* src_str_desc_ptr {};// %string_descriptor
+                dest_str_desc_loaded = builder->CreateLoad(llvm_data_type->getPointerTo(), first_ptr);
+                src_str_desc_ptr = builder->CreateLoad(llvm_data_type, builder->CreateLoad(llvm_data_type->getPointerTo(), src_data_ptr));
+                builder->CreateStore(src_str_desc_ptr, dest_str_desc_loaded);
+            } else { // e.g. `{ %f64*, i32, %dimension_descriptor*, i1, i32 }`
+                builder->CreateStore(builder->CreateLoad(llvm_data_type->getPointerTo(), src_data_ptr), first_ptr);
+            }
 
-            // Data pointer has been moved to dest, so set src's data pointer to null
-            builder->CreateStore(llvm::ConstantPointerNull::get(llvm_data_type->getPointerTo()), src_data_ptr);
+            // Data pointer has been moved to dest, so set src's data pointer to null            
+            if(ASRUtils::is_character(*asr_data_type)){ // `{ %string_descriptor*, i32, %dimension_descriptor*, i1, i32 }` -- Keep `string_descriptor` but clear its state.
+                llvm::Value* src_data_ptr = llvm_utils->get_stringArray_data(asr_data_type /*Type Of Src*/, src, true);
+                builder->CreateStore(llvm::ConstantPointerNull::get(llvm_utils->character_type), src_data_ptr);
+            } else { // e.g. `{ %f64*, i32, %dimension_descriptor*, i1, i32 }`
+                builder->CreateStore(llvm::ConstantPointerNull::get(llvm_data_type->getPointerTo()), src_data_ptr);
+            }
 
             llvm::Value* src_offset = this->get_offset(dest_ty, src);
             llvm::Value* dest_offset = this->get_offset(dest_ty, dest, false);
