@@ -11,10 +11,10 @@ namespace LCompilers {
     std::string save_pycfile(const ASR::TranslationUnit_t &m, LCompilers::LocationManager lm);
 
     // Load a module from a modfile
-    ASR::TranslationUnit_t* load_modfile(Allocator &al, const std::string &s,
+    Result<ASR::TranslationUnit_t*, ErrorMessage> load_modfile(Allocator &al, const std::string &s,
         bool load_symtab_id, SymbolTable &symtab, LCompilers::LocationManager &lm);
 
-    ASR::TranslationUnit_t* load_pycfile(Allocator &al, const std::string &s,
+    Result<ASR::TranslationUnit_t*, ErrorMessage> load_pycfile(Allocator &al, const std::string &s,
         bool load_symtab_id, LCompilers::LocationManager &lm);
 
 } // namespace LCompilers
