@@ -159,6 +159,11 @@ class FixTypeVisitor: public ASR::CallReplacerOnExpressionsVisitor<FixTypeVisito
         visit_ArrayOp(x);
     }
 
+    void visit_ComplexBinOp(const ASR::ComplexBinOp_t& x) {
+        ASR::CallReplacerOnExpressionsVisitor<FixTypeVisitor>::visit_ComplexBinOp(x);
+        visit_ArrayOp(x);
+    }
+
     void visit_RealCompare(const ASR::RealCompare_t& x) {
         ASR::CallReplacerOnExpressionsVisitor<FixTypeVisitor>::visit_RealCompare(x);
         visit_ArrayOp(x);
@@ -251,17 +256,42 @@ class ReplaceArrayOp: public ASR::BaseExprReplacer<ReplaceArrayOp> {
         LCOMPILERS_ASSERT(result_expr != nullptr);
         ASR::ttype_t* result_type = ASRUtils::expr_type(result_expr);
         ASR::ttype_t* result_element_type = ASRUtils::extract_type(result_type);
-        for( int64_t i = 0; i < ASRUtils::get_fixed_size_of_array(x->m_type); i++ ) {
+
+        ASR::dimension_t* m_dims = nullptr;
+        size_t n_dims = ASRUtils::extract_dimensions_from_ttype(x->m_type, m_dims);
+
+        int64_t size = ASRUtils::get_fixed_size_of_array(x->m_type);
+        int repeat = 1;
+        std::vector<std::vector<int64_t>> rank_indexes(n_dims, std::vector<int64_t>(size));
+        for (size_t i = 0; i < n_dims; i++) {
+            int64_t length = 0, start = 1, ubound;
+            ASRUtils::extract_value(m_dims[i].m_length, length);
+            ASRUtils::extract_value(m_dims[i].m_start, start);
+            ubound = length + start - 1;
+            int64_t c = 0;
+            while (c != size) {
+                for (int64_t j = start; j <= ubound; j++) {
+                    for (int64_t k = 1; k <= repeat; k++) {
+                        rank_indexes[i][c++] = j;
+                    }
+                }
+            }
+            repeat *= length;
+        }
+
+        for (int64_t i = 0; i < size; i++) {
             ASR::expr_t* x_i = ASRUtils::fetch_ArrayConstant_value(al, x, i);
             Vec<ASR::array_index_t> array_index_args;
-            array_index_args.reserve(al, 1);
-            ASR::array_index_t array_index_arg;
-            array_index_arg.loc = loc;
-            array_index_arg.m_left = nullptr;
-            array_index_arg.m_right = make_ConstantWithKind(
-                make_IntegerConstant_t, make_Integer_t, i + 1, 4, loc);
-            array_index_arg.m_step = nullptr;
-            array_index_args.push_back(al, array_index_arg);
+            array_index_args.reserve(al, n_dims);
+            for (size_t j = 0; j < n_dims; j++) {
+                ASR::array_index_t array_index_arg;
+                array_index_arg.loc = loc;
+                array_index_arg.m_left = nullptr;
+                array_index_arg.m_right = make_ConstantWithKind(
+                    make_IntegerConstant_t, make_Integer_t, rank_indexes[j][i], 4, loc);
+                array_index_arg.m_step = nullptr;
+                array_index_args.push_back(al, array_index_arg);
+            }
             ASR::expr_t* y_i = ASRUtils::EXPR(ASRUtils::make_ArrayItem_t_util(al, loc,
                 result_expr, array_index_args.p, array_index_args.size(),
                 result_element_type, ASR::arraystorageType::ColMajor, nullptr));
@@ -339,7 +369,6 @@ class ReplaceArrayOp: public ASR::BaseExprReplacer<ReplaceArrayOp> {
             ASR::array_index_t array_index_arg;
             array_index_arg.loc = loc;
             array_index_arg.m_left = nullptr;
-            // TODO: Make this work with any rank
             array_index_arg.m_right = builder.Add(m_dims[0].m_start, make_ConstantWithKind(
                 make_IntegerConstant_t, make_Integer_t, i, 4, loc));
             array_index_arg.m_step = nullptr;
@@ -361,6 +390,63 @@ ASR::expr_t* at(Vec<ASR::expr_t*>& vec, int64_t index) {
     return vec[index];
 }
 
+// Collect the array expressions which should be of the same size
+class CollectComponentsFromElementalExpr: public ASR::BaseWalkVisitor<CollectComponentsFromElementalExpr> {
+private:
+        Allocator& al;
+        Vec<ASR::expr_t*>& vars;
+public:
+
+        CollectComponentsFromElementalExpr(Allocator& al_, Vec<ASR::expr_t*>& vars_) :
+        al(al_), vars(vars_) {}
+
+        void push_expr_if_array(ASR::expr_t *v) {
+            if (ASRUtils::is_array(ASRUtils::expr_type(v))) {
+                vars.push_back(al, v);
+            }
+        }
+
+        // Don't go inside these
+        void visit_ttype(const ASR::ttype_t &) {}
+        void visit_ArraySection(const ASR::ArraySection_t&) {}
+        void visit_ArrayItem(const ASR::ArrayItem_t&) {}
+        void visit_ArraySize(const ASR::ArraySize_t&) {}
+        void visit_ArrayReshape(const ASR::ArrayReshape_t&) {}
+        void visit_ArrayBound(const ASR::ArrayBound_t&) {}
+
+        void visit_Var(const ASR::Var_t& x) {
+            ASR::Var_t *xx = const_cast<ASR::Var_t*>(&x);
+            push_expr_if_array((ASR::expr_t *)xx);
+        }
+
+        void visit_ArrayPhysicalCast(const ASR::ArrayPhysicalCast_t& x) {
+            ASR::ArrayPhysicalCast_t *xx = const_cast<ASR::ArrayPhysicalCast_t*>(&x);
+            push_expr_if_array((ASR::expr_t *)xx);
+        }
+
+        void visit_StructInstanceMember(const ASR::StructInstanceMember_t& x) {
+            ASR::StructInstanceMember_t *xx = const_cast<ASR::StructInstanceMember_t*>(&x);
+            push_expr_if_array((ASR::expr_t *)xx);
+        }
+
+        void visit_BitCast(const ASR::BitCast_t& x) {
+            ASR::BitCast_t *xx = const_cast<ASR::BitCast_t*>(&x);
+            push_expr_if_array((ASR::expr_t *)xx);
+        }
+
+        void visit_ArrayConstant(const ASR::ArrayConstant_t& x) {
+            ASR::ArrayConstant_t *xx = const_cast<ASR::ArrayConstant_t*>(&x);
+            push_expr_if_array((ASR::expr_t *)xx);
+        }
+
+        // Only go inside the FunctionCall if the Function is elemental
+        void visit_FunctionCall(const ASR::FunctionCall_t& x) {
+            if (ASRUtils::is_elemental(x.m_name)) {
+                ASR::BaseWalkVisitor<CollectComponentsFromElementalExpr>::visit_FunctionCall(x);
+            }
+        }
+};
+
 class ArrayOpVisitor: public ASR::CallReplacerOnExpressionsVisitor<ArrayOpVisitor> {
     private:
 
@@ -958,6 +1044,16 @@ class ArrayOpVisitor: public ASR::CallReplacerOnExpressionsVisitor<ArrayOpVisito
         (void)x;
     }
 
+    bool is_looping_necessary_for_bitcast(ASR::expr_t* value) {
+        if (ASR::is_a<ASR::BitCast_t>(*value)) {
+            ASR::BitCast_t* bit_cast = ASR::down_cast<ASR::BitCast_t>(value);
+            return !ASRUtils::is_string_only(ASRUtils::expr_type(bit_cast->m_source));
+        } else {
+            return false;
+        }
+    }
+
+
     void visit_Assignment(const ASR::Assignment_t& x) {
         if (ASRUtils::is_simd_array(x.m_target)) {
             if( !(ASRUtils::is_allocatable(x.m_value) ||
@@ -1018,7 +1114,7 @@ class ArrayOpVisitor: public ASR::CallReplacerOnExpressionsVisitor<ArrayOpVisito
         var_collector_value.current_expr = const_cast<ASR::expr_t**>(&(xx.m_value));
         var_collector_value.call_replacer();
 
-        if (vars.size() == 1 &&
+        if (vars.size() == 1 && !is_looping_necessary_for_bitcast(xx.m_value) && 
             ASRUtils::is_array(ASRUtils::expr_type(ASRUtils::get_past_array_broadcast(xx.m_value)))
         ) {
             return ;
@@ -1032,17 +1128,17 @@ class ArrayOpVisitor: public ASR::CallReplacerOnExpressionsVisitor<ArrayOpVisito
             ASRUtils::is_array(ASRUtils::expr_type(x.m_target)) &&
             ASRUtils::is_array(ASRUtils::expr_type(x.m_value))) {
             ASRUtils::ExprStmtDuplicator expr_duplicator(al);
-            ASR::expr_t* d_target = ASRUtils::get_expr_size_expr(expr_duplicator.duplicate_expr(x.m_target));
-            ASR::expr_t* d_orig_value = expr_duplicator.duplicate_expr(x.m_value);
-            ASR::expr_t* d_value = ASRUtils::get_expr_size_expr(expr_duplicator.duplicate_expr(x.m_value));
+            ASR::expr_t* d_target = expr_duplicator.duplicate_expr(x.m_target);
+            ASR::expr_t* d_value = expr_duplicator.duplicate_expr(x.m_value);
+
+            Vec<ASR::expr_t*> vars;
+            vars.reserve(al, 1);
+
+            CollectComponentsFromElementalExpr cv(al, vars);
+            cv.visit_expr(*d_value);
 
-            ASR::ttype_t *type32 = ASRUtils::TYPE(ASR::make_Integer_t(al, x.base.base.loc, 4));
-            ASR::expr_t* d_target_size = ASRUtils::EXPR(ASR::make_ArraySize_t(al, x.base.base.loc,
-                d_target, nullptr, type32, nullptr));
-            ASR::expr_t* d_value_size = ASRUtils::EXPR(ASR::make_ArraySize_t(al, x.base.base.loc,
-                d_value, nullptr, type32, nullptr));
             if (debug_inserted.find(&x) == debug_inserted.end()) {
-                pass_result.push_back(al, ASRUtils::STMT(ASR::make_DebugCheckArrayBounds_t(al, x.base.base.loc, d_target_size, d_value_size, d_orig_value, x.m_move_allocation)));
+                pass_result.push_back(al, ASRUtils::STMT(ASR::make_DebugCheckArrayBounds_t(al, x.base.base.loc, d_target, vars.p, vars.n, x.m_move_allocation)));
                 if (!x.m_move_allocation) {
                     debug_inserted.insert(&x);
                 }
