@@ -20,7 +20,6 @@ inline std::string get_intrinsic_name(int64_t x) {
     switch (x) {
         INTRINSIC_NAME_CASE(ObjectType)
         INTRINSIC_NAME_CASE(Kind)
-        INTRINSIC_NAME_CASE(Rank)
         INTRINSIC_NAME_CASE(Sin)
         INTRINSIC_NAME_CASE(Cos)
         INTRINSIC_NAME_CASE(Tan)
@@ -152,6 +151,7 @@ inline std::string get_intrinsic_name(int64_t x) {
         INTRINSIC_NAME_CASE(Popcnt)
         INTRINSIC_NAME_CASE(Poppar)
         INTRINSIC_NAME_CASE(Real)
+        INTRINSIC_NAME_CASE(Cmplx)
         INTRINSIC_NAME_CASE(Dim)
         INTRINSIC_NAME_CASE(Anint)
         INTRINSIC_NAME_CASE(Sqrt)
@@ -397,8 +397,6 @@ namespace IntrinsicElementalFunctionRegistry {
             {&Hypot::instantiate_Hypot, &Hypot::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Kind),
             {nullptr, &Kind::verify_args}},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Rank),
-            {nullptr, &Rank::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Digits),
             {&Digits::instantiate_Digits, &Digits::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Rrspacing),
@@ -463,6 +461,8 @@ namespace IntrinsicElementalFunctionRegistry {
             {&Poppar::instantiate_Poppar, &Poppar::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Real),
             {&Real::instantiate_Real, &Real::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Cmplx),
+            {&Cmplx::instantiate_Cmplx, &Cmplx::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Nint),
             {&Nint::instantiate_Nint, &Nint::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Idnint),
@@ -770,8 +770,6 @@ namespace IntrinsicElementalFunctionRegistry {
             "selected_char_kind"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Kind),
             "kind"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Rank),
-            "rank"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Digits),
             "Digits"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Rrspacing),
@@ -832,6 +830,8 @@ namespace IntrinsicElementalFunctionRegistry {
             "poppar"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Real),
             "real"},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Cmplx),
+            "cmplx"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Nint),
             "nint"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Idnint),
@@ -1021,7 +1021,6 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"selected_real_kind", {&SelectedRealKind::create_SelectedRealKind, &SelectedRealKind::eval_SelectedRealKind}},
                 {"selected_char_kind", {&SelectedCharKind::create_SelectedCharKind, &SelectedCharKind::eval_SelectedCharKind}},
                 {"kind", {&Kind::create_Kind, &Kind::eval_Kind}},
-                {"rank", {&Rank::create_Rank, &Rank::eval_Rank}},
                 {"digits", {&Digits::create_Digits, &Digits::eval_Digits}},
                 {"rrspacing", {&Rrspacing::create_Rrspacing, &Rrspacing::eval_Rrspacing}},
                 {"repeat", {&Repeat::create_Repeat, &Repeat::eval_Repeat}},
@@ -1062,6 +1061,7 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"popcnt", {&Popcnt::create_Popcnt, &Popcnt::eval_Popcnt}},
                 {"poppar", {&Poppar::create_Poppar, &Poppar::eval_Poppar}},
                 {"real", {&Real::create_Real, &Real::eval_Real}},
+                {"cmplx", {&Cmplx::create_Cmplx, &Cmplx::eval_Cmplx}},
                 {"nint", {&Nint::create_Nint, &Nint::eval_Nint}},
                 {"idnint", {&Idnint::create_Idnint, &Idnint::eval_Idnint}},
                 {"anint", {&Anint::create_Anint, &Anint::eval_Anint}},
