@@ -76,18 +76,25 @@ class ReplacePresentCalls: public ASR::BaseExprReplacer<ReplacePresentCalls> {
 
     void replace_IntrinsicElementalFunction(ASR::IntrinsicElementalFunction_t* x) {
         if (x->m_intrinsic_id == static_cast<int64_t>(ASRUtils::IntrinsicElementalFunctions::Present)) {
+            if (!ASR::is_a<ASR::Var_t>(*x->m_args[0])) {
+                return;
+            }
             ASR::symbol_t* present_arg = ASR::down_cast<ASR::Var_t>(x->m_args[0])->m_v;
             size_t i;
             for( i = 0; i < f->n_args; i++ ) {
-                if( ASR::down_cast<ASR::Var_t>(f->m_args[i])->m_v == present_arg ) {
-                    i++;
-                    break;
+                if( ASR::down_cast<ASR::Var_t>(f->m_args[i])->m_v ) {
+                    if( ASR::down_cast<ASR::Var_t>(f->m_args[i])->m_v == present_arg ) {
+                        i++;
+                        break;
+                    }
                 }
             }
 
-            *current_expr = ASRUtils::EXPR(ASR::make_Var_t(al, x->base.base.loc,
-                                ASR::down_cast<ASR::Var_t>(f->m_args[i])->m_v));
-            return;
+            if (i < f->n_args) {
+                *current_expr = ASRUtils::EXPR(ASR::make_Var_t(al, x->base.base.loc,
+                                    ASR::down_cast<ASR::Var_t>(f->m_args[i])->m_v));
+                return;
+            }
         }
         for (size_t i = 0; i < x->n_args; i++) {
             ASR::expr_t** current_expr_copy_12 = current_expr;
@@ -344,8 +351,8 @@ bool fill_new_args(Vec<ASR::call_arg_t>& new_args, Allocator& al,
     }
     bool is_nopass { false };
     bool is_class_procedure { false };
-    if (ASR::is_a<ASR::ClassProcedure_t>(*func_sym)) {
-        ASR::ClassProcedure_t* class_proc = ASR::down_cast<ASR::ClassProcedure_t>(func_sym);
+    if (ASR::is_a<ASR::StructMethodDeclaration_t>(*func_sym)) {
+        ASR::StructMethodDeclaration_t* class_proc = ASR::down_cast<ASR::StructMethodDeclaration_t>(func_sym);
         func_sym = class_proc->m_proc;
         is_nopass = class_proc->m_is_nopass;
         is_class_procedure = true;
@@ -370,28 +377,38 @@ bool fill_new_args(Vec<ASR::call_arg_t>& new_args, Allocator& al,
         return false;
     }
 
-    // when `func` is a ClassProcedure **without** nopass, then the
+    // when `func` is a StructMethodDeclaration **without** nopass, then the
     // first argument of FunctionType is "this" (i.e. the class instance)
     // which is depicted in `func.n_args` while isn't depicted in
     // `x.n_args` (as it only represents the "FunctionCall" arguments)
     // hence to adjust for that, `is_method` introduces an offset
-    bool is_method = is_class_procedure && (!is_nopass);
+    int is_method = is_class_procedure && (!is_nopass);
 
     new_args.reserve(al, func->n_args);
-    for( size_t i = 0, j = 0; j < func->n_args; j++, i++ ) {
+    for( int i = 0, j = 0; j < (int)func->n_args; j++, i++ ) {
         if( std::find(sym2optionalargidx[func_sym].begin(),
                       sym2optionalargidx[func_sym].end(), j)
             != sym2optionalargidx[func_sym].end() ) {
             ASR::Variable_t* func_arg_j = ASRUtils::EXPR2VAR(func->m_args[j]);
-            if( i - is_method >= x.n_args || x.m_args[i - is_method].m_value == nullptr ) {
+            if( i - is_method >= (int)x.n_args || x.m_args[i - is_method].m_value == nullptr ) {
                 std::string m_arg_i_name = scope->get_unique_name("__libasr_created_variable_");
                 ASR::ttype_t* arg_type = func_arg_j->m_type;
+                if(ASR::is_a<ASR::String_t>(*ASRUtils::extract_type(arg_type))){// Create String type with dummy len info.
+                    arg_type = ASRUtils::duplicate_type(al, arg_type); // New-duplicated node
+                    ASR::String_t* str = ASR::down_cast<ASR::String_t>(
+                        ASRUtils::extract_type(arg_type));
+                    if( str->m_len &&
+                        !ASR::is_a<ASR::IntegerConstant_t>(*str->m_len)){
+                        str->m_len = ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, arg_type->base.loc, 0,
+                            ASRUtils::TYPE(ASR::make_Integer_t(al, arg_type->base.loc, 4))));
+                    }
+                }
                 ASR::symbol_t* arg_decl = func_arg_j->m_type_declaration;
-                if( ASR::is_a<ASR::Array_t>(*arg_type) ) {
+                if( ASR::is_a<ASR::Array_t>(*arg_type) ) { // Create dummy array dims
                     ASR::Array_t* array_t = ASR::down_cast<ASR::Array_t>(arg_type);
                     Vec<ASR::dimension_t> dims;
                     dims.reserve(al, array_t->n_dims);
-                    for( size_t i = 0; i < array_t->n_dims; i++ ) {
+                    for( int i = 0; i < (int)array_t->n_dims; i++ ) {
                         ASR::dimension_t dim;
                         dim.m_length = ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, arg_type->base.loc, 1,
                                             ASRUtils::TYPE(ASR::make_Integer_t(al, arg_type->base.loc, 4))));
@@ -401,10 +418,11 @@ bool fill_new_args(Vec<ASR::call_arg_t>& new_args, Allocator& al,
                         dims.push_back(al, dim);
                     }
                     arg_type = ASRUtils::TYPE(ASR::make_Array_t(al, arg_type->base.loc,
-                                array_t->m_type, dims.p, dims.size(), ASR::array_physical_typeType::FixedSizeArray));
+                                array_t->m_type, dims.p, dims.size(),
+                                ASRUtils::is_character(*array_t->m_type)? ASR::PointerArray : ASR::FixedSizeArray));
                 }
                 ASR::expr_t* m_arg_i = PassUtils::create_auxiliary_variable(
-                    x.m_args[i - is_method].loc, m_arg_i_name, al, scope, arg_type, ASR::intentType::Local, arg_decl);
+                    x.m_args[i - is_method].loc, m_arg_i_name, al, scope, arg_type, ASR::intentType::Local, arg_decl, func->m_args[j]);
                 arg_type = ASRUtils::expr_type(m_arg_i);
                 if( ASRUtils::is_array(arg_type) &&
                     ASRUtils::extract_physical_type(arg_type) !=
@@ -425,7 +443,7 @@ bool fill_new_args(Vec<ASR::call_arg_t>& new_args, Allocator& al,
             ASR::ttype_t* logical_t = ASRUtils::TYPE(ASR::make_Logical_t(al,
                                         x.m_args[i - is_method].loc, 4));
             ASR::expr_t* is_present = nullptr;
-            if( i - is_method >= x.n_args || x.m_args[i - is_method].m_value == nullptr ) {
+            if( i - is_method >= (int)x.n_args || x.m_args[i - is_method].m_value == nullptr ) {
                 is_present = ASRUtils::EXPR(ASR::make_LogicalConstant_t(
                     al, x.m_args[0].loc, false, logical_t));
             } else {
@@ -459,13 +477,23 @@ bool fill_new_args(Vec<ASR::call_arg_t>& new_args, Allocator& al,
                 }
 
                 if( is_present == nullptr ) {
-                    is_present = ASRUtils::EXPR(ASR::make_LogicalConstant_t(
-                        al, x.m_args[i - is_method].loc, true, logical_t));
+                    ASR::expr_t* arg_i = x.m_args[i - is_method].m_value;
+                    const Location& loc = arg_i->base.loc;
+                    LCOMPILERS_ASSERT(arg_i != nullptr);
+                    if( ASRUtils::is_pointer(ASRUtils::expr_type(arg_i)) ) {
+                        ASR::ttype_t* associated_type_ = ASRUtils::TYPE(
+                            ASR::make_Logical_t(al, loc, 4));
+                        is_present = ASRUtils::EXPR(ASR::make_PointerAssociated_t(
+                            al, loc, arg_i, nullptr, associated_type_, nullptr));
+                    } else {
+                        is_present = ASRUtils::EXPR(ASR::make_LogicalConstant_t(
+                            al, loc, true, logical_t));
+                    }
                 }
             }
             ASR::call_arg_t present_arg;
             present_arg.loc = x.m_args[i - is_method].loc;
-            if( i - is_method < x.n_args && 
+            if( i - is_method < (int)x.n_args &&
                 x.m_args[i - is_method].m_value &&
                 ASRUtils::is_allocatable(x.m_args[i - is_method].m_value) &&
                 !ASRUtils::is_allocatable(func_arg_j->m_type) ) {
@@ -478,7 +506,8 @@ bool fill_new_args(Vec<ASR::call_arg_t>& new_args, Allocator& al,
             present_arg.m_value = is_present;
             new_args.push_back(al, present_arg);
             j++;
-        } else if (!is_method) {
+        } else {
+            if(i - is_method < 0) continue;
             // not needed to have `i - is_method` can be simply
             // `i` as well, just for consistency with code above
             new_args.push_back(al, x.m_args[i - is_method]);
@@ -487,7 +516,7 @@ bool fill_new_args(Vec<ASR::call_arg_t>& new_args, Allocator& al,
     }
     // new_args.size() is either
     //      - equal to func->n_args
-    //      - one less than func->n_args (in case of ClassProcedure without nopass)
+    //      - one less than func->n_args (in case of StructMethodDeclaration without nopass)
     LCOMPILERS_ASSERT(func->n_args == new_args.size() + is_method);
     return true;
 }
@@ -518,7 +547,7 @@ class ReplaceFunctionCallsWithOptionalArguments: public ASR::BaseExprReplacer<Re
         *current_expr = ASRUtils::EXPR(ASRUtils::make_FunctionCall_t_util(al,
                             x->base.base.loc, x->m_name, x->m_original_name,
                             new_args.p, new_args.size(), x->m_type, x->m_value,
-                            x->m_dt, ASRUtils::get_class_proc_nopass_val((*x).m_name)));
+                            x->m_dt));
         new_func_calls.insert(*current_expr);
     }
 
@@ -567,7 +596,7 @@ class ReplaceSubroutineCallsWithOptionalArgumentsVisitor : public PassUtils::Pas
             pass_result.push_back(al, ASRUtils::STMT(ASRUtils::make_SubroutineCall_t_util(al,
                                     x.base.base.loc, x.m_name, x.m_original_name,
                                     new_args.p, new_args.size(), x.m_dt,
-                                    nullptr, false, ASRUtils::get_class_proc_nopass_val(x.m_name))));
+                                    nullptr, false)));
         }
 };
 
