@@ -8,9 +8,9 @@ namespace LCompilers {
 
     // Pickle an ASR node
     std::string pickle(ASR::asr_t &asr, bool colors=false, bool indent=false,
-            bool show_intrinsic_modules=false);
+            bool show_intrinsic_modules=false, bool clojure=false);
     std::string pickle(ASR::TranslationUnit_t &asr, bool colors=false,
-            bool indent=false, bool show_intrinsic_modules=false);
+            bool indent=false, bool show_intrinsic_modules=false, bool clojure=false);
 
     // Print the tree structure
     std::string pickle_tree(ASR::asr_t &asr, bool colors, bool show_intrinsic_modules=false);
