@@ -12,6 +12,7 @@ enum Platform {
     Linux,
     macOS_Intel,
     macOS_ARM,
+    macOS_PowerPC,
     Windows,
     FreeBSD,
     OpenBSD,
@@ -67,6 +68,7 @@ struct PassOptions {
     bool time_report = false;
     bool skip_removal_of_unused_procedures_in_pass_array_by_data = false;
     bool bounds_checking = true;
+    bool strict_bounds_checking = false;
     std::vector<std::string> vector_of_time_report;
 };
 
@@ -99,7 +101,7 @@ struct CompilerOptions {
     std::string column = "";
     bool continue_compilation = false;
     bool semantics_only = false;
-    bool new_classes = false;
+    bool new_classes = true;
     /*
         Generates object code for modules as well as global procedures ( subroutines / functions )
         avialable in ASR. This needs to be explicity set to true.
