@@ -19,5 +19,16 @@ integer, parameter :: character_kinds(1) = [1]
 integer, parameter :: logical_kinds(1) = [4]
 
 integer, parameter :: iostat_end = -1
+integer, parameter :: iostat_eor = -2
+
+integer, parameter :: numeric_storage_size = 32
+integer, parameter :: character_storage_size = 8
+
+contains
+function compiler_version() result(version)
+    character(len=:), allocatable :: version
+    version = _lfortran_compiler_version() ! note: LFortran takes this and creates an IntrinsicElementalFunction
+end function compiler_version
+
 
 end module
